/*************************************************************************

 I/O functions

 ---------------------------------------------------------------------

 Copyright (c) 2005, 2006
 Carlos M. Fonseca <cmfonsec@ualg.pt>
 Manuel Lopez-Ibanez <manuel.lopez-ibanez@ulb.ac.be>
 Luis Paquete <paquete@dei.uc.pt>

 This program is free software (software libre); you can redistribute
 it and/or modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2 of the
 License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, you can obtain a copy of the GNU
 General Public License at:
 http://www.gnu.org/copyleft/gpl.html
 or by writing to:
 Free Software Foundation, Inc., 59 Temple Place,
 Suite 330, Boston, MA 02111-1307 USA

 ----------------------------------------------------------------------

 *************************************************************************/

#include "HVio.h"
#include "string.h" /* strerror */
#include "errno.h" /* errno */

#define PAGE_SIZE 4096          /* allocate one page at a time      */
#define DATA_INC (PAGE_SIZE/sizeof(double))

void read_data(population * pop, double **data_p, int *nobjs_p, int size,
		int* sort, int without) {
	int size2;
	int i,j;
	int nobjs = *nobjs_p; /* number of objectives (and columns).  */
	double *data = *data_p;
	int ntotal = 0;

	if (pop == NULL) {
		errprintf(" population is empty !\n");
		exit(EXIT_FAILURE);
	}

	data = realloc(data, size * nobjs * sizeof(double));

	size2 = size;
	if (without!=-1) size2++;
	// printf("hypervol :\n");
	for (i = 0; i < size2; i++) {
		if (i != without) {
			for (j = 0; j < nobjs; j++) {
				data[ntotal] = pop->ind[sort[i]].obj[j];
				// printf("%f ",data[ntotal]);
				ntotal++;
			}
	//		printf("\n");
		}
	}

	*data_p = data;
}

/* From:

 Edition 0.10, last updated 2001-07-06, of `The GNU C Library
 Reference Manual', for Version 2.3.x.

 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2001, 2002,
 2003 Free Software Foundation, Inc.
 */
void errprintf(const char *template, ...) {
	extern char *program_invocation_short_name;
	va_list ap;

	fprintf(stderr, "%s: error: ", program_invocation_short_name);
	va_start(ap,template);
	vfprintf(stderr, template, ap);
	va_end(ap);
	fprintf(stderr, "\n");

	exit(EXIT_FAILURE);
}
/* End of copyright The GNU C Library Reference Manual */

void warnprintf(const char *template, ...) {
	extern char *program_invocation_short_name;
	va_list ap;

	fprintf(stderr, "%s: warning: ", program_invocation_short_name);
	va_start(ap,template);
	vfprintf(stderr, template, ap);
	va_end(ap);
	fprintf(stderr, "\n");

}
