/*
 * affin.c
 *
 *  Created on: Mar 1, 2011
 *      Author: Thomas PIERRARD
 */
# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <assert.h>

# include "datastruct.h"

void define_affinity(population *antg, population *antb) {
	int i, j, k, rndindex, index, current_rank;
	double leftn, rightn;
	double maxab, max, min;
	double range;

	maxab = 0.0;
	// Calculate affinity for antibodies
	if (antb->size>0) {
		if (antg->size > 0) {
			// Calculate affinity (euclidean distance) between all antibodies and one antigen
			rndindex = rnd(0, antg->size - 1);
			for (i = 0; i < antb->size; i++) {
				double eucld = eucl_d(&(antg->ind[rndindex]), &(antb->ind[i]))+1.0;
				antb->ind[i].affinity = 1.0/eucld;
				/*printf("1/eucld : eucld : %f, antb aff : %f\n", eucl_d(&(antg->ind[rndindex]),
				 &(antb->ind[i])), antb->ind[i].affinity);
				 */
			}
		} else {
			for (i = 0; i < antb->size; i++) {
				antb->ind[i].affinity = 1.0/(antb->ind[i].rank+1.0);
			}
		}
		
		// Find the better antibody
		for (i = 0; i < antb->size; i++)
			if (antb->ind[i].affinity>maxab) maxab =  antb->ind[i].affinity;
		
		
		for (i = 0; i < antb->size; i++) sort[i] = i;
		myqsort_rank(antb, 0, antb->size - 1, sort);

		// find bounds of antibodies
		// and assigned them to the maximum 
		// affinity found so far for antibodies
		for (i = 0; i < nobj; i++) {
			max = -INF;
			j = 0;
			index = -1;
			//printf("rank : %d\n", antb->ind[sort[0]].rank);
			current_rank = antb->ind[sort[j]].rank;
			//printf("rank : %d\n", current_rank);
			while (j<antb->size) {
				if (antb->ind[sort[j]].rank!=current_rank) break; 
				if (antb->ind[sort[j]].obj[i]>max) {
					max = antb->ind[sort[j]].obj[i];
					index = j;
				}
				j++;
				//printf("j : %d, sort[j] : %d, antb->size : %d\n",j,sort[j], antb->size);
			}
			if (antb->size>0) {
				assert(index!=-1);
				antb->ind[sort[index]].affinity = maxab;
			}
		}
	}


	if (antg->size > 2) {
		for (k = 0; k < antg->size; k++) antg->ind[k].affinity = 1.0; 
		for (i = 0; i < nobj; i++) {
			
			//find range and minimum for normalization :
			min = INF;
			max = -INF;
			for (k = 0; k < antg->size; k++) {
				if (antg->ind[k].obj[i]<min) min = antg->ind[k].obj[i];
				if (antg->ind[k].obj[i]>max) max = antg->ind[k].obj[i];
			}
			range = max-min;
			//printf("range : %f %f %f\n", range, min, max);
			// ---------------------------------------			
			for (k = 0; k < antg->size; k++) sort[k] = k;
			myqsort(antg, 0, antg->size - 1, i, sort);
			for (j = 1; j < antg->size - 1; j++) {
				
				if (range==0.0) leftn = 0.0;
				else leftn = ((antg->ind[sort[j]].obj[i])-min)/range;

				if (range==0.0) rightn = 0.0;
				else
					rightn = ((antg->ind[sort[j + 1]].obj[i])-min)/range;
				antg->ind[sort[j]].affinity *= (double)absreal(rightn - leftn);
				
			}
		}
		max = -INF;
		for (k = 0; k < antg->size; k++) {
			antg->ind[k].affinity += maxab;
			if (antg->ind[k].affinity>max) max = antg->ind[k].affinity;
		}

		for (i = 0; i < nobj; i++) {
			for (k = 0; k < antg->size; k++) sort[k] = k;
			myqsort(antg, 0, antg->size - 1, i, sort);
			antg->ind[sort[antg->size - 1]].affinity = max;
		}

	} else {
		for(i=0;i<antg->size;i++)
			antg->ind[i].affinity = maxab+0.5*maxab;
	}
}
