/*
 * clmut.c
 *
 *  Created on: Mar 1, 2011
 *      Author: Thomas PIERRARD
 */
# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <assert.h>
# include "datastruct.h"

// Select candidates with greater affinity and clone them
void clone_pop(population * antb, population * antg, population * pool) {
	int i, j;
	int nb_ext_clone, nb_nml_clone, tobeclone, nb_candidate;
	double sum_aff;
	population *candidate;
	
	// Candidate population to perform cloning
	candidate = (population *) malloc(sizeof(population));
	allocate_memory_pop(candidate, popsize);

	// Compute total numbers of clones for extreme candidates and normal candidates 
	nb_nml_clone = (int) (((double)curgen/(double)ngen) * 0.4*popsize) + 0.5*popsize;
	nb_ext_clone = popsize - nb_nml_clone;
	//printf("res : %d %d %d\n",nb_ext_clone,nb_nml_clone,popsize);
	assert((nb_ext_clone+nb_nml_clone)==popsize);

	// Add individuals in the candidate population sorted by affinity
	candidate->size = 0;

	for (i=0; i<antg->size;i++) sort[i] = i;
	myqsort_affinity_inv(antg, 0, antg->size-1, sort);

	for (i = 0; i < antg->size; i++)
		copy_ind(&(antg->ind[sort[i]]), &(candidate->ind[candidate->size++]));

	for (i=0; i<antb->size;i++) sort[i] = i;
	myqsort_affinity_inv(antb, 0, antb->size-1, sort);

	for (i = 0; i < antb->size; i++) {
		copy_ind(&(antb->ind[sort[i]]), &(candidate->ind[candidate->size++]));
	}
	
	pool->size = 0;
	// For the first nobjth individuals, clone them such that 
	// the total number of clones is "nb_ext_clone"
	tobeclone = ceil((nb_ext_clone)/ nobj);
	//printf("tobeclone_ext : %d\n",tobeclone);
	for (i=0;i<nobj;i++) {
		for (j=0;j<tobeclone;j++) {
			copy_ind(&(candidate->ind[i]), &(pool->ind[pool->size++]));
		}
	}
	//printf("poolsize : %d\n",pool->size);
	// For the rest of the individuals, clone X times them such that
	// the number of clone is proportional to their affinity.
	nb_candidate = (int) (nb_cl*0.3*popsize+0.1*popsize);
	//printf("nb_candidate : %d\n", nb_candidate);
	sum_aff = 0.0;
	// Compute sum of affinity of remaining individuals
	for (i=nobj;i<nb_candidate;i++) sum_aff += candidate->ind[i].affinity;
	for (i=nobj;i<nb_candidate && pool->size<popsize;i++) {
		// define number of clones for candidate i :
		tobeclone = ceil(nb_nml_clone*(candidate->ind[i].affinity)/sum_aff);
		//printf("tobeclone : %d\n",tobeclone);
		for (j=0;j<tobeclone && pool->size<popsize;j++) {
			copy_ind(&(candidate->ind[i]), &(pool->ind[pool->size++]));
		}
		//printf("poolsize : %d, i :%d, nb_cand: %d\n",pool->size, i, nb_candidate);
		if ((i==nb_candidate-1) && (pool->size<popsize)) {
			while (pool->size<popsize) {
				copy_ind(&(candidate->ind[i]), &(pool->ind[pool->size++]));
			}
		}	
	}

	assert(pool->size==popsize);
	deallocate_memory_pop(candidate, popsize);
	free(candidate);

}


/* Mutate all the individuals of population pop */
void mutat_pop(population * pop) {
	int i;
	double param, x, p_mut_type;

	x = -6.0 + ((double)curgen/(double)ngen) * 12.0; 
	param = -4.0 + st_gs * 8.0;
	p_mut_type = 1.0/(1.0+exp(-2.0*(x+param)));

	//printf("mut type : %f\n", p_mut_type);
	for (i = 0; i < pop->size; i++) {

		if (randomperc() > p_mut_type)
			mutat_ind_gauss_global(&(pop->ind[i]));
		else
			mutat_ind_gauss_local(&(pop->ind[i]));
	}
}

/* Mutate an individual : Gaussian mutation */
void mutat_ind_gauss_local(individual *ind) {
	int i, atleastone;
	double range, bxm;
	double lrange, rrange;
	atleastone = rnd(0, nreal - 1);

	for (i = 0; i < nreal; i++) {
		if (atleastone == i || (randomperc() <= (1.0/(double)nreal))) {
			range = max_realvar[i] - min_realvar[i];
			rrange = max_realvar[i] - ind->xreal[i];
			lrange = ind->xreal[i] - min_realvar[i];
			atleastone = -1;
			bxm = 0.1*box_muller(0.0, mut_stl);
			//printf("gaussian local : bxm:%f \n", bxm);
			ind->xreal[i] = ind->xreal[i] + bxm * range;


			if (ind->xreal[i] < min_realvar[i]) {
				ind->xreal[i] = min_realvar[i];
			} else {
				if (ind->xreal[i] > max_realvar[i])
					ind->xreal[i] = max_realvar[i];
			}
	//		printf("gaussian : %f\n", mut_p);

		}
	}
}

/* Mutate an individual : Gaussian mutation */
void mutat_ind_gauss_global(individual *ind) {
	int i, atleastone;
	double range, bxm;
	double lrange, rrange;
	atleastone = rnd(0, nreal - 1);

	for (i = 0; i < nreal; i++) {
		if (atleastone == i || (randomperc() <= (1.0/(double)nreal))) {
			range = max_realvar[i] - min_realvar[i];
			rrange = max_realvar[i] - ind->xreal[i];
			lrange = ind->xreal[i] - min_realvar[i];
			atleastone = -1;
			bxm = 0.1*box_muller(0.0, mut_stg);
			//printf("gaussian_global : bxm:%f\n", bxm);
			ind->xreal[i] = ind->xreal[i] + bxm * range;

			if (ind->xreal[i] < min_realvar[i]) {
				ind->xreal[i] = min_realvar[i];
			} else {
				if (ind->xreal[i] > max_realvar[i])
					ind->xreal[i] = max_realvar[i];
			}

		}
	}
}


