/*
 * datastruct.h
 *
 *  Created on: Feb 1, 2011
 *      Author: Thomas PIERRARD
 */
# define PI 3.14159265358979
# define INF 1.0e14
# define GNUPLOT_COMMAND "gnuplot -persist"

#define RESET		0
#define BRIGHT 		1
#define DIM		2
#define UNDERLINE 	3
#define BLINK		4
#define REVERSE		7
#define HIDDEN		8

#define BLACK 		0
#define RED		1
#define GREEN		2
#define YELLOW		3
#define BLUE		4
#define MAGENTA		5
#define CYAN		6
#define	WHITE		7

/* structure of an individual */
typedef struct {
	double *obj;
	double affinity;

} individual;

/* structure of a population */
typedef struct {
	individual *ind;
	int size;
} population;

/* Variables of the algorithm */
extern int nobj;
extern int popsize;
extern int *sort;

/* Variables of the random engine */
extern double seed;
extern double oldrand[55];
extern int jrand;

/* necessary because popen and pclose are not ansi functions */
extern FILE *popen(const char *command, const char *modes);
extern int pclose(FILE *stream);

/* alloc.c : Memory allocation */
void allocate_memory_pop(population *pop, int size);
void allocate_memory_ind(individual *ind);
void deallocate_memory_pop(population *pop, int size);
void deallocate_memory_ind(individual *ind);

/* domin.c : Dominance of an individal */
int dominate(individual *a, individual *b);

/* funct.c : Auxilliary functions */
double eucl_d(individual * a, individual * b);
double absreal(double a);
int same(individual * a, individual * b);
void swap(int *x, int *y);

/* HV*.c files : Signature of the function needed ? */
double hv_compute(population * pop, double **reference, double *maximum,
		double *minimum, int size, int *nobj_p);

/* qsort.c : Quick sort algorithms */
void myqsort(population *pop, int m, int n, int obj, int * sort);
void myqsort_affinity(population *pop, int m, int n, int * sort);
void myqsort_affinity_inv(population *pop, int m, int n, int * sort);

/* rando.c : Random engine */
void randomize(void);
void warmup_random(double seed);
void advance_random(void);
double randomperc(void);
int rnd(int low, int high);
double rndreal(double low, double high);

