/*
 * gplot.c
 *
 *  Created on: Mar 30, 2011
 *      Author: Thomas PIERRARD
 */

/* Routines to display the population information using gnuplot */

#include <stdio.h>
#include <stdlib.h>

#include "datastruct.h"

void display_all_pop() {
	int i;
	char * fn1, *fn2;
	FILE * fgp_png;
	for (i = 10; i <= ngen; i+=10) {
		display_gen(i);
		fn1 = malloc(50 * sizeof(char));
		fn2 = malloc(50 * sizeof(char));
		sprintf(fn1, "gnuplot_output/%s/%s%i_antg.png", prob_name, prob_name, i);
		sprintf(fn2, "gnuplot_output/%s/%s%i.png", prob_name, prob_name, i);
		/* waiting for png file creation */
		while (!(fgp_png = fopen(fn1, "r"))) {
		}
		fclose(fgp_png);
		while (!(fgp_png = fopen(fn2, "r"))) {
		}
		fclose(fgp_png);
		free(fn1);
		free(fn2);
	}
	return;
}

void display_gen(int ii) {

	FILE * fpt, *gp, *gp2;
	double max_x, max_y, max_z;
	double min_x, min_y, min_z;
	char * fn, *fn2, *fn3, *fn4, *fn5;
	double x, y, z;
	int flag, flag1, flag2, flag3;

	max_x = -INF;
	max_y = -INF;
	max_z = -INF;
	min_x = INF;
	min_y = INF;
	min_z = INF;

	gp = popen(GNUPLOT_COMMAND, "w");
	if (gp == NULL) {
		printf(
				"\n Could not open a pipe to gnuplot, check the definition of GNUPLOT_COMMAND in file global.h\n");
		printf(
				"\n Edit the string to suit your system configuration and rerun the program\n");
		exit(1);
	}

	gp2 = popen(GNUPLOT_COMMAND, "w");
	if (gp2 == NULL) {
		printf(
				"\n Could not open a pipe to gnuplot, check the definition of GNUPLOT_COMMAND in file global.h\n");
		printf(
				"\n Edit the string to suit your system configuration and rerun the program\n");
		exit(1);
	}
	flag = 1;
	fn = malloc(50 * sizeof(char));
	sprintf(fn, "plot_out/antigens_%d.out", ii);

	if (nobj == 3) {
		fpt = fopen(fn, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\t%le\n", &x, &y, &z) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
				max_z = (z > max_z) ? z : max_z;
				min_z = (z < min_z) ? z : min_z;
			}
			fclose(fpt);
		} else {
			flag = 0;
		}
	} else {
		fpt = fopen(fn, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\n", &x, &y) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
			}
			fclose(fpt);
		} else {
			flag = 0;
		}
	}
	if (flag) {
		FILE * gnup;
		gnup = fopen("gnuplot.config", "w");
		fprintf(gnup, "set terminal png size 800,600\n");
		fprintf(gnup, "set output \"gnuplot_output/%s/%s%i_antg.png\"\n",
				prob_name, prob_name, ii);
		fclose(gnup);
		fprintf(gp, "load 'gnuplot.config'\n");

		if (nobj != 3) {
			fprintf(
					gp,
					"set title 'Generation #%d'\n set xrange [%f:%f]\n set yrange [%f:%f]\n",
					ii, min_x, max_x, min_y, max_y);
			fprintf(
					gp,
					"plot \"%s\" title \'Antigens\' w points pointtype 6 pointsize 1\n",
					fn);

		} else {
			fprintf(
					gp,
					"set title 'Generation #%d'\n set view %d,%d\n set nokey\n set xrange [%f:%f]\n set yrange [%f:%f]\n set zrange [%f:%f]\n",
					ii, 30, 60, min_x, max_x, min_y, max_y, min_z, max_z);
			fprintf(
					gp,
					"splot '%s' ti \"Antigens\" w points pointtype 6 pointsize 1\n",
					fn);
		}
	}
	fflush(gp);
	pclose(gp);
	free(fn);

	fn2 = malloc(50 * sizeof(char));
	fn3 = malloc(50 * sizeof(char));
	fn4 = malloc(50 * sizeof(char));
	fn5 = malloc(50 * sizeof(char));

	sprintf(fn2, "plot_out/clones_%d.out", ii);
	sprintf(fn3, "plot_out/antibodies_%d.out", ii);
	sprintf(fn4, "plot_out/antigens_%d.out", ii);
	sprintf(fn5, "plot_out/candidates_%d.out", ii);

	flag1 = 1;
	flag2 = 1;
	flag3 = 1;

	max_x = -INF;
	max_y = -INF;
	max_z = -INF;
	min_x = INF;
	min_y = INF;
	min_z = INF;

	if (nobj == 3) {
		fpt = fopen(fn2, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\t%le\n", &x, &y, &z) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
				max_z = (z > max_z) ? z : max_z;
				min_z = (z < min_z) ? z : min_z;
			}
			fclose(fpt);
		}
		fpt = fopen(fn3, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\t%le\n", &x, &y, &z) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
				max_z = (z > max_z) ? z : max_z;
				min_z = (z < min_z) ? z : min_z;
			}
			fclose(fpt);
		} else {
			flag1 = 0;
		}
		fpt = fopen(fn4, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\t%le\n", &x, &y, &z) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
				max_z = (z > max_z) ? z : max_z;
				min_z = (z < min_z) ? z : min_z;
			}
			fclose(fpt);
		} else {
			flag2 = 0;
		}
		fpt = fopen(fn5, "r");
		if (!fpt) {
			flag3 = 0;
		} else {
			fclose(fpt);
		}
	} else {
		fpt = fopen(fn2, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\n", &x, &y) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
			}
			fclose(fpt);
		}
		fpt = fopen(fn3, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\n", &x, &y) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
			}
			fclose(fpt);
		} else {
			flag1 = 0;
		}
		fpt = fopen(fn4, "r");
		if (fpt) {
			while (fscanf(fpt, "%le\t%le\n", &x, &y) != EOF) {
				max_x = (x > max_x) ? x : max_x;
				min_x = (x < min_x) ? x : min_x;
				max_y = (y > max_y) ? y : max_y;
				min_y = (y < min_y) ? y : min_y;
			}
			fclose(fpt);
		} else {
			flag2 = 0;
		}
		fpt = fopen(fn5, "r");
		if (!fpt) {
			flag3 = 0;
		} else {
			fclose(fpt);
		}
	}

	FILE * gnup2;
	gnup2 = fopen("gnuplot2.config", "w");
	fprintf(gnup2, "set terminal png size 800,600\n");
	fprintf(gnup2, "set output \"gnuplot_output/%s/%s%i.png\"\n", prob_name,
			prob_name, ii);
	fclose(gnup2);
	fprintf(gp2, "load 'gnuplot2.config'\n");

	if (nobj != 3) {
		fprintf(
				gp2,
				"set title 'Generation #%d'\n set xrange [%f:%f]\n set yrange [%f:%f]\n",
				ii, min_x, max_x, min_y, max_y);
		fprintf(
				gp2,
				"plot \"%s\" title \'Clones\' w points pointtype 6 pointsize 1",
				fn2);
		if (flag1)
			fprintf(gp2, ", \"%s\" ti \"Antibodies\"", fn3);
		if (flag2)
			fprintf(gp2, ", \"%s\" ti \"Antigens\"", fn4);
		if (flag3)
			fprintf(gp2, ", \"%s\" ti \"Candidates\"", fn5);
		fprintf(gp2, "\n");

	} else {
		fprintf(
				gp2,
				"set title 'Generation #%d'\n set view %d,%d\n set nokey\n set xrange [%f:%f]\n set yrange [%f:%f]\n set zrange [%f:%f]\n",
				ii, 30, 60, min_x, max_x, min_y, max_y, min_z, max_z);
		fprintf(gp2,
				"splot '%s' ti \"Clones\" w points pointtype 6 pointsize 1",
				fn2);
		if (flag1)
			fprintf(gp2, ", \"%s\" ti \"Antibodies\"", fn3);
		if (flag2)
			fprintf(gp2, ", \"%s\" ti \"Antigens\"", fn4);
		if (flag3)
			fprintf(gp2, ", \"%s\" ti \"Candidates\"", fn5);
		fprintf(gp2, "\n");

	}
	fflush(gp2);
	pclose(gp2);
	free(fn2);
	free(fn3);
	free(fn4);
	free(fn5);
	return;
}

