/*
 * moisa.c
 *
 *  Created on: Mar 1, 2011
 *      Author: Thomas PIERRARD
 */
# include <stdio.h>
# include <stdlib.h>
# include <dlfcn.h>
# include <unistd.h>
# include <assert.h>
# include <math.h>
# include <sys/time.h>

#include "datastruct.h"

/* algorithm variables */
int nreal;
int nobj;
int ncon;
int ngen;
int curgen;
int popsize;
int nbeval;
int *sort;
double *min_realvar;
double *max_realvar;

/* parameters */
double st_gs;
double nb_cl;
double mut_stg;
double mut_stl;
double mut_step;
double p_mut_type;

/* debug */
int showtime = 0;
int display_antb = 0;
int display_antg = 0;
int display_main = 0;

/* options */
char* prob_name;
int gp_png;
globalArgs_t globalArgs;
int retval;
void * plugin;

int main(int argc, char **argv) {
	/* Local variable */
	int i, j;
	FILE *res;
	double finlh;
	population * pop, *antb, *antg, *pool;

	/* To count function evaluations */
	nbeval = 0;

	/* Parsing options */
	parseopts(argc, argv);

	/* Step-size parameter */
	mut_stg = 0.5+mut_stg*1.5;
	mut_stl = 0.1+mut_stl*0.4;

	// Array to sort individuals by ranks, first objective...
	sort = (int *) malloc(2 * popsize * sizeof(int));

	// Main population
	pop = (population *) malloc(sizeof(population));
	allocate_memory_pop(pop, popsize);

	// Pool population
	pool = (population *) malloc(sizeof(population));
	allocate_memory_pop(pool, 2 * popsize);

	// Antibodies population
	antb = (population *) malloc(sizeof(population));
	allocate_memory_pop(antb, popsize);

	// Antigens population
	antg = (population *) malloc(sizeof(population));
	allocate_memory_pop(antg, popsize);

	// Initialize the pseudo-random system
	randomize();

	// Initialize the population
	initialize_pop(pop);

	curgen = 0;
	printf("Starting generations...\n");
	printf("-> gen : %d    \r", curgen++);
	fflush(stdout);

	// Evaluation of the main population (objective+constraints)
	evaluate_pop(pop);

	// Non-dominated ranking on the initial population
	dominated_sort(pop);

	// Split main population into two sets
	split(pop, antb, antg);
	//output_pop(antg, curgen, "antigens");
	//output_pop(antb, curgen, "antibodies");

	while (curgen < ngen) {
		printf("-> gen : %d    \r", curgen++);
		fflush(stdout);

		// Define affinity for each antibody and antigens
		define_affinity(antg, antb);

		// DISPLAY ANTIGENS (pop, start_gen, end_gen, curr_gen)
		//for(j=0;j<antg->size;j++) sort[j] = j;
		//myqsort_affinity_inv(antg, 0, antg->size - 1, sort);
		//display_pop_sorted(antg, 1, ngen, curgen, "antigens[obj]");
		output_pop(antg, curgen, "antigens");

		//for (i = 0; i < antg->size; i++) sort[i] = i;
		//finlh = hv_compute(antg, NULL, NULL, NULL, antg->size, sort, -1, &nobj);
		//printf("1. Antg only : %f\n", finlh);


		// DISPLAY ANTIBODIES (pop, start_gen, end_gen, curr_gen)
		//for(j=0;j<antb->size;j++) sort[j] = j;
		//myqsort_affinity_inv(antb, 0, antb->size - 1, sort);
		//display_pop_sorted(antb, 1, ngen, curgen, "antibodies[obj]");
		output_pop(antb, curgen, "antibodies");

		// Clone the candidate in a pool
		clone_pop(antb, antg, pool);

		// Mutate the clones
		mutat_pop(pool);

		// Evaluate the clones
		evaluate_pop(pool);
		//output_pop(pool, curgen, "clones");
			
		// Merge old pop and clones
		//printf("pool size : %d\n",pool->size);
		//for (i = 0; i < pool->size; i++) sort[i] = i;
		//finlh = hv_compute(pool, NULL, NULL, NULL, pool->size, sort, -1, &nobj);
		//printf("2. Pool only : %f\n", finlh);
		merge(pool, antg);

		// Rank the merge population
		//for (i = 0; i < pool->size; i++) sort[i] = i;
		//finlh = hv_compute(pool, NULL, NULL, NULL, pool->size, sort, -1, &nobj);
		//printf("3. Pool & Antg : %f\n", finlh);
		dominated_sort(pool);

		// Take the best individuals of the pool to form the new pop.
		if (antg->size==popsize && curgen > (ngen*2/3)) {
			discard2(pop,pool, antg);
		//	printf("curgen : %d\n", curgen);
			}
		else
			discard(pop, pool);

		//for (i = 0; i < pop->size; i++) sort[i] = i;
		//finlh = hv_compute(pop, NULL, NULL, NULL, pop->size, sort, -1, &nobj);
		//printf("4. Pop : %f\n", finlh);
		//output_pop(pop, curgen, "newpop");

		// Split main population into two sets
		split(pop, antb, antg);
		//assert((antg->size+antb->size)==popsize);

	}
	/*
	 for (i = 0; i < antg->size; i++) {
	 sort[i] = i;
	 }
	 double finlh = hv_compute(antg, NULL, NULL, NULL, antg->size, sort, -1,
	 &nobj);
	 */
	res = fopen("final_pop.out", "w");
	for (i = 0; i < antg->size; i++) {
		if (nobj == 2)
			fprintf(res, "%e\t%e\n", antg->ind[i].obj[0], antg->ind[i].obj[1]);
		else
			fprintf(res, "%e\t%e\t%e\n", antg->ind[i].obj[0],
					antg->ind[i].obj[1], antg->ind[i].obj[2]);
		fflush(res);

	}
	fclose(res);
	printf("Number evaluations : %d\n", nbeval);
	//printf("\nGenerating graphics...\n");
	//display_all_pop();
	//printf("prob, seed, prec, hv : %s %f %f %f\n",prob_name, seed, precision, finlh);

	deallocate_memory_pop(pop, popsize);
	free(pop);
	deallocate_memory_pop(antg, popsize);
	free(antg);
	deallocate_memory_pop(antb, popsize);
	free(antb);
	deallocate_memory_pop(pool, popsize * 2);
	free(pool);
	free(sort);
	free(min_realvar);
	free(max_realvar);
	dlclose(plugin);
	return 0;
}

