/*
 * parse.c
 *
 *  Created on: Mar 1, 2011
 *      Author: Thomas PIERRARD
 */
# include <stdio.h>
# include <stdlib.h>
# include <dlfcn.h>
# include <unistd.h>

# include "global.h"
# include "rand.h"

static const char *optString = "s:p:h?";

void display_usage() {
	printf("\n Usage ./nsga2r -s random_seed -p specific_problem [-h?]\n"
		"-h or -? displays this help\n");
	exit(1);
}

int parseopts(int argc, char * argv[]) {

	char cmd[100];
	int resultat; /* value returned by gcc */
	char * error; /* error message returned by dlerror */
	int opt = 0;

	opt = getopt(argc, argv, optString);
	while (opt != -1) {
		switch (opt) {
		case 's':
			globalArgs.seed = (double) atof(optarg);
			break;
		case 'p':
			globalArgs.probname = (char *) optarg;
			break;
		case 'h': /* fall-through is intentional */
		case '?':
			display_usage();
			break;
		default:
			display_usage();
			break;
		}

		opt = getopt(argc, argv, optString);
	}

	/* problem_name definition as an extern variable */
	prob_name = (char *) globalArgs.probname;
	seed = globalArgs.seed;

	/* deleting previous files */
	char * fn = malloc(80 * sizeof(char));
	sprintf(fn, "rm -Rf plot_out/* *.so *.out gnuplot_output/%s/*", prob_name);
	retval = system(fn);
	free(fn);

	/* do we output to png? */
	char * fn1, *fn2;
	FILE * fnpt1, *fnpt2;
	if (!(fnpt1 = fopen("gnuplot_output", "r")))
		retval = system("mkdir gnuplot_output/");
	else
		fclose(fnpt1);
	fn1 = malloc(30 * sizeof(char));
	fn2 = malloc(30 * sizeof(char));
	sprintf(fn1, "gnuplot_output/%s", prob_name);
	sprintf(fn2, "mkdir gnuplot_output/%s", prob_name);
	if (!(fnpt2 = fopen(fn1, "r")))
		retval = system(fn2);
	else
		fclose(fnpt2);

	if (globalArgs.seed <= 0.0 || globalArgs.seed >= 1.0) {
		printf("\n Entered seed value is wrong, seed value must be in (0,1) \n");
		exit(1);
	}

	/******************************        PLUGIN             **************************/

	/* compilation of the plugin */
	sprintf(cmd, "gcc -shared -fPIC -o ./plugin.so lib/%s.c -lm", prob_name);
	resultat = system(cmd);
	if (!
	WIFEXITED(resultat)) {
		fprintf(stderr, "Erreur pendant la compilation de %s.c.\n"
			" gcc s'est arrete de maniere inattendue\n", prob_name);
		exit(1);
	} else if (
	WEXITSTATUS(resultat)) {
		fprintf(stderr, "Erreur pendant la compilation de %s.c.\n"
			"Code de retour de gcc : %d\n", prob_name,
		WEXITSTATUS(resultat));
		exit(1);
	}

	/* plugin loading */
	if ((plugin = dlopen("./plugin.so", RTLD_NOW)) == NULL) {
		fprintf(stderr, "Error while loading %s.so : %s\n", prob_name,
				dlerror());
		fprintf(stderr, "Did you use an unknown symbol ?\n");
		exit(1);
	}
	printf("\nProblem %s has been loaded\n", prob_name);

	/* Recuperation des pointeurs sur les fonctions */
	dlerror(); /* Effacer une eventuelle precedente erreur */

	*(void **) (&test_problem) = dlsym(plugin, "problem_def");

	/* get the plugin variable */

	if ((error = dlerror()) != NULL) {
		fprintf(stderr, "Error while loading function : %s\n", error);
		exit(1);
	}

	/****************************************************************************************************************/
	free(fn1);
	free(fn2);
	return EXIT_SUCCESS;
}
