/*
 * parse.c
 *
 *  Created on: Mar 1, 2011
 *      Author: Thomas PIERRARD
 */
# include <stdio.h>
# include <stdlib.h>
# include <dlfcn.h>
# include <wait.h>
# include <unistd.h>

# include "datastruct.h"

static const char *optString = "s:p:a:b:c:d:h?";

void display_usage() {
	printf(
			"\nUsage : \n./moisa -s seed -p problem [-a ggm -b cnc -c mss -d num]  [-h?]\n"
				"s : Seed, ]0,1[\n"
				"p : Problem name, [zdt1, kur, tnk, dtlz3 ...]\n"
				"a : Trade-off Global vs Local search, [0,1]\n"
				"b : Step-size Global search, [0,1]\n",
				"c : Step-size Local search, [0,1]\n",
				"d : Number of candidate to clone, [0,1]\n");
	exit(1);
}

void check_gnuplot() {
	char * fn1, *fn2;
	FILE * fnpt1, *fnpt2;
	if (!(fnpt1 = fopen("gnuplot_output", "r")))
		retval = system("mkdir gnuplot_output/");
	else
		fclose(fnpt1);
	fn1 = malloc(30 * sizeof(char));
	fn2 = malloc(30 * sizeof(char));
	sprintf(fn1, "gnuplot_output/%s", prob_name);
	sprintf(fn2, "mkdir gnuplot_output/%s", prob_name);
	if (!(fnpt2 = fopen(fn1, "r")))
		retval = system(fn2);
	else
		fclose(fnpt2);
	free(fn1);
	free(fn2);
}

int parseopts(int argc, char * argv[]) {

	char cmd[256], line[256];
	int i, result; /* value returned by gcc */
	char *error, *fn;
	int opt, flags, flagp;
	FILE * param;

	flags = 0;
	flagp = 0;

	globalArgs.st_gs = 0.8;
	globalArgs.mut_stg = 0.8;
	globalArgs.mut_stl = 0.8;
	globalArgs.nb_cl = 0.5;

	opt = getopt(argc, argv, optString);
	while (opt != -1) {
		switch (opt) {
		case 's':
			globalArgs.seed = (double) atof(optarg);
			flags = 1;
			break;
		case 'p':
			globalArgs.probname = (char *) optarg;
			flagp = 1;
			break;
		case 'a':
			globalArgs.st_gs = (double) atof(optarg);
			break;
		case 'b':
			globalArgs.mut_stg = (double) atof(optarg);
			break;
		case 'c':
			globalArgs.mut_stl = (double) atof(optarg);
			break;
		case 'd':
			globalArgs.nb_cl = (double) atof(optarg);
			break;
		default:
			display_usage();
			break;
		}
		opt = getopt(argc, argv, optString);
	}
	if ((!flags) || (!flagp)) {
		printf("\nSeed and Problem name are two mandatory parameters!");
		display_usage();
	}
	if (globalArgs.seed < 0.0 || globalArgs.seed > 1.0) {
		printf("\n Entered seed value is wrong, seed value must be in ]0,1[ \n");
		display_usage();
	}
	if (globalArgs.st_gs < 0.0 || globalArgs.st_gs > 1.0) {
		printf("\n Entered st_gs value is wrong, st_gs value must be in [0,1] \n");
		display_usage();
	}
	if (globalArgs.mut_stg < 0.0 || globalArgs.mut_stg > 1.0) {
		printf("\n Entered mut_stg value is wrong, mut_stg value must be in [0,1] \n");
		display_usage();
	}
	if (globalArgs.mut_stl < 0.0 || globalArgs.mut_stl > 1.0) {
		printf("\n Entered mut_stl value is wrong, mut_stl value must be in [0,1] \n");
		display_usage();
	}
	if (globalArgs.nb_cl < 0.0 || globalArgs.nb_cl > 1.0) {
		printf("\n Entered nb_cl value is wrong, cl_nb value must be in [0,1] \n");
		display_usage();
	}

	/* problem_name definition as an extern variable */
	prob_name = (char *) globalArgs.probname;
	seed = globalArgs.seed;
	st_gs = globalArgs.st_gs;
	mut_stg = globalArgs.mut_stg;
	mut_stl = globalArgs.mut_stl;
	nb_cl = globalArgs.nb_cl;

	/* deleting previous files */
	fn = malloc(80 * sizeof(char));
	sprintf(fn, "rm -Rf plot_out/* *.so *.out gnuplot_output/%s/*", prob_name);
	retval = system(fn);
	free(fn);

	/* make sure folder for gnuplot is created */
	check_gnuplot();

	/******************************        PLUGIN             **************************/

	sprintf(cmd, "gcc -shared -fPIC -o ./plugin.so lib/%s.c -lm", prob_name);
	result = system(cmd);
	if (!
	WIFEXITED(result)) {
		fprintf(stderr, "Error while compiling %s.c.\n", prob_name);
		exit(1);
	} else if (
	WEXITSTATUS(result)) {
		fprintf(stderr, "Error while compiling %s.c.\n"
			"Gcc return code : %d\n", prob_name,
		WEXITSTATUS(result));
		exit(1);
	}

	/* plugin loading */
	if ((plugin = dlopen("./plugin.so", RTLD_NOW)) == NULL) {
		fprintf(stderr, "Error while loading %s.so : %s\n", prob_name,
				dlerror());
		fprintf(stderr, "Did you use an unknown symbol ?\n");
		exit(1);
	}
	printf("Problem %s has been loaded\n", prob_name);

	dlerror();

	*(void **) (&test_problem) = dlsym(plugin, "problem_def");

	if ((error = dlerror()) != NULL) {
		fprintf(stderr, "Error while loading function : %s\n", error);
		exit(1);
	}

	/************************************ END PLUGIN  *********************************************/

	fn = malloc(30 * sizeof(char));
	sprintf(fn, "input_data/%s.in", prob_name);
	param = fopen(fn, "r");
	free(fn);
	if (!param) {
		printf("Error : Can't open file %s.in", prob_name);
		exit(1);
	}

	error = fgets(line, 256, param);
	// Population
	retval = sscanf(line, "%d", &popsize);
	if (popsize < 40) {
		printf("\n Wrong population size entered\n");
		exit(1);
	}
	printf("Population size :%d\n", popsize);

	error = fgets(line, 256, param);
	// Number of generation
	retval = sscanf(line, "%d", &ngen);
	if (ngen < 1) {
		printf("\n Wrong number of generations entered\n");
		exit(1);
	}
	printf("Number of generation :%d\n", ngen);

	error = fgets(line, 256, param);
	// Number of objectives
	retval = sscanf(line, "%d", &nobj);
	if (nobj < 1) {
		printf("\n Wrong number of objectives entered\n");
		exit(1);
	}
	printf("Number of objective :%d\n", nobj);

	error = fgets(line, 256, param);
	// Number of constraints
	retval = sscanf(line, "%d", &ncon);
	if (ncon < 0) {
		printf("\n Wrong number of constraints entered\n");
		exit(1);
	}
	printf("Number of constraints :%d\n", ncon);

	error = fgets(line, 256, param);
	// Number of real variables
	retval = sscanf(line, "%d", &nreal);
	if (nreal < 0) {
		printf("\n Wrong number of variables entered\n");
		exit(1);
	}
	printf("Number of variables :%d\n", nreal);

	if (nreal != 0) {
		min_realvar = (double *) malloc(nreal * sizeof(double));
		max_realvar = (double *) malloc(nreal * sizeof(double));
		for (i = 0; i < nreal; i++) {
			error = fgets(line, 256, param);
			retval = sscanf(line, "%lf %lf", &min_realvar[i], &max_realvar[i]);
			if (max_realvar[i] <= min_realvar[i]) {
				printf(
						"\n Wrong limits entered for the min and max bounds of real variable\n");
				exit(1);
			}
		}
	}
	fclose(param);
	/*
	 // Precision parameter
	 retval = scanf("%d", &precision);
	 if (precision < 0 || precision > 50) {
	 printf("\n Wrong number for precision entered\n");
	 exit(1);
	 }
	 printf("Precision :%d\n", precision);


	 // Beta parameter
	 retval = scanf("%d", &beta);
	 if (beta < 0 || beta > 5) {
	 printf("\n Wrong number for beta entered\n");
	 exit(1);
	 }
	 printf("Beta :%d\n", beta);
	 */

	return EXIT_SUCCESS;
}
